;;
;; Relocating ABC80 .bac loader loading at BOFA
;;
;; To load a device driver, move BOFA to the desired address
;; after initialization, move EOFA to the same address as BOFA
;; and put a 01h byte at the new EOFA (this does a NEW).
;;
;; To load an application that can be RUN more than once, prepend
;; to the payload data:
;;
;; .defb 0x0b, 0x01, 0x00, 0x84, 0xc7
;; .defw ENTRYPOINT		; Put relocation here
;; .defb 0xce, 0x3a, 0xba, 0x0d, 0x01
;;

	include "setup80.inc"

_start:
	ld bc,END		; END entry point in BASIC (B = 0!)
	push bc			; Save as return address
	ld hl,(BOFA)		; BOFA = start of data
	push hl			; Save for exit
	push hl			; Save for relocation
	ld e,l
	ld d,h
	ld c,(hl)		; First BASIC line = BASIC prefix
	add hl,bc		; Skip prefix

	include "copy80.inc"

set_reloc_len:
	defc NEG_RELOC_LEN=set_reloc_len+1
	ld hl,0			; Negative (length of relocation data + 1)
	add hl,de

	;;
	;; Move EOFA to the end of our data
	;; This is probably unnecessary but easy to do
	;; The C code will already have inserted a 01h byte to this address
	;;
	ld (EOFA),hl
	inc hl			; HL -> relocations
	pop de			; DE -> data (= BOFA)

	;; Set a delta if needed.  This loader only supports an
	;; 8-bit delta
	ld a,e
setdelta:
	defc DELTA=setdelta+1
	add a,0
	ld c,a
	ld a,d
	adc a,0
	ld b,a

	include "relocate.inc"

	; All done, jump to entry point
	; Note: top of stack is 00C9h -> END, so a RET from the
	; target routine if the same as END.
done:
	defc ENTRY=done+1
	ld bc,0
	pop hl			; HL -> load address (= BOFA)
	add hl,bc
	jp (hl)

	include "end80.inc"
